#ifndef CAR_PHYSICS_HPP
#define CAR_PHYSICS_HPP

namespace godot {
class CarPhysics : public RigidBody3D {
    GDCLASS(CarPhysics, RigidBody3D);
    static void _bind_methods();
public:
    virtual void _enter_tree() override;
    virtual void _integrate_forces(PhysicsDirectBodyState3D *state) override;
    void set_target_speed(float target);
    float get_target_speed() const;
    void set_current_steering(float steering);
    float get_current_steering() const;
    void set_brake(bool value);
    bool get_brake() const;
    void set_oversteer_curve(Ref<Curve> curve);
    Ref<Curve> get_oversteer_curve() const;
    void set_understeer_curve(Ref<Curve> curve);
    Ref<Curve> get_understeer_curve() const;
private:
    float target_speed{0.f};
    float current_steering{0.f};
    Vector3 local_velocity{};
    float gravity_velocity{-1.f};
    bool brake{false};
    Ref<Curve> oversteer_curve{};
    Ref<Curve> understeer_curve{};

    static const float ACCELERATION;
    static const float BRAKE_FORCE;
    static const float UNDERSTEER_SPEED_MUL;
    static const float OVERSTEER_SPEED_MUL;
};
}

#endif // !CAR_PHYSICS_HPP
